import contextlib
import unittest
import io
import re
import unittest.mock
from unittest import TextTestRunner

class MainTest(unittest.TestCase):

    def test_answer(self):
        capturedOutput = io.StringIO()                 
        sys.stdout = capturedOutput                     
        main()
        sys.stdout = sys.__stdout__ 

        checkQuery = [
            'PAN AMS CAS NYC HEL 504.5 kg',
            'PAN AMS CAS HEL NYC 486.5 kg',
            'PAN AMS NYC CAS HEL 520.7 kg',
            'PAN AMS NYC HEL CAS 560.5 kg',
            'PAN AMS HEL CAS NYC 435.9 kg',
            'PAN AMS HEL NYC CAS 493.6 kg',
            'PAN CAS AMS NYC HEL 495.6 kg',
            'PAN CAS AMS HEL NYC 410.8 kg',
            'PAN CAS NYC AMS HEL 445.0 kg',
            'PAN CAS NYC HEL AMS 475.2 kg',
            'PAN CAS HEL AMS NYC 427.1 kg',
            'PAN CAS HEL NYC AMS 542.0 kg',
            'PAN NYC AMS CAS HEL 350.5 kg',
            'PAN NYC AMS HEL CAS 339.7 kg',
            'PAN NYC CAS AMS HEL 283.7 kg',
            'PAN NYC CAS HEL AMS 330.1 kg',
            'PAN NYC HEL AMS CAS 323.5 kg',
            'PAN NYC HEL CAS AMS 380.7 kg',
            'PAN HEL AMS CAS NYC 421.6 kg',
            'PAN HEL AMS NYC CAS 495.5 kg',
            'PAN HEL CAS AMS NYC 488.4 kg',
            'PAN HEL CAS NYC AMS 552.7 kg',
            'PAN HEL NYC AMS CAS 546.1 kg',
            'PAN HEL NYC CAS AMS 536.5 kg'
        ]
        modelAnswerLength = len(checkQuery)

        output = list(capturedOutput.getvalue().strip().split("\n"))
        if output[-1] == "":
            output.pop()

        

        self.assertNotEqual(len(output), 120, "You are printing all possible route combinations. That's not what's expected.")

        # Check output format
        # Start: 5 ports with 3 routes each, separated by a space
        # End: with x.x kg
        r = re.compile('^((\w{3} ){5})(\d+.\d kg)$') 
        isFormatCorrect = all(r.match(line) for line in output)
        self.assertTrue(isFormatCorrect, "The output format is incorrect. Please take a look at the example output.")
        
        r = re.compile('HEL (\d+.\d kg)') # Finds routes with "HEL x.x kg"
        alwaysEndsWithHel = all(r.match(line) for line in output)
        self.assertFalse(alwaysEndsWithHel, "The output always ends with HEL. You might have some incorrect indentation.")
        
        startsWithPan = bool(all(route[:3] == 'PAN' for route in output))
        self.assertTrue(startsWithPan, "Remember to start the routes with Panama!")

        self.assertFalse(len(output) > modelAnswerLength, "Too many route combinations. Are you missing some constraints?")
        self.assertFalse(len(output) < modelAnswerLength, "Too few route combinations. Are you missing some routes?")
        
        # Default Test
        # Check if output and checkQuery contain the same elements irrespective of order
        self.assertTrue(sorted(output) == sorted(checkQuery), "Your answer is incorrect.")

        # self.assertEqual(capturedOutput.getvalue().strip(), 'PAN AMS CAS NYC HEL 504.5 kg\nPAN AMS CAS HEL NYC 486.5 kg\nPAN AMS NYC CAS HEL 520.7 kg\nPAN AMS NYC HEL CAS 560.5 kg\nPAN AMS HEL CAS NYC 435.9 kg\nPAN AMS HEL NYC CAS 493.6 kg\nPAN CAS AMS NYC HEL 495.6 kg\nPAN CAS AMS HEL NYC 410.8 kg\nPAN CAS NYC AMS HEL 445.0 kg\nPAN CAS NYC HEL AMS 475.2 kg\nPAN CAS HEL AMS NYC 427.1 kg\nPAN CAS HEL NYC AMS 542.0 kg\nPAN NYC AMS CAS HEL 350.5 kg\nPAN NYC AMS HEL CAS 339.7 kg\nPAN NYC CAS AMS HEL 283.7 kg\nPAN NYC CAS HEL AMS 330.1 kg\nPAN NYC HEL AMS CAS 323.5 kg\nPAN NYC HEL CAS AMS 380.7 kg\nPAN HEL AMS CAS NYC 421.6 kg\nPAN HEL AMS NYC CAS 495.5 kg\nPAN HEL CAS AMS NYC 488.4 kg\nPAN HEL CAS NYC AMS 552.7 kg\nPAN HEL NYC AMS CAS 546.1 kg\nPAN HEL NYC CAS AMS 536.5 kg')

test_suite = unittest.TestLoader().loadTestsFromTestCase(MainTest)
with io.StringIO() as buf:
    # run the tests
    with contextlib.redirect_stdout(buf):
        TextTestRunner(stream=buf).run(test_suite)
    # process (in this case: print) the results
    printTest(buf.getvalue())